# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: auth_common.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x61uth_common.proto\x12\x18\x61rmonik.api.grpc.v1.auth\"<\n\x04User\x12\x10\n\x08username\x18\x01 \x01(\t\x12\r\n\x05roles\x18\x02 \x03(\t\x12\x13\n\x0bpermissions\x18\x03 \x03(\t\"\x17\n\x15GetCurrentUserRequest\"F\n\x16GetCurrentUserResponse\x12,\n\x04user\x18\x01 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.auth.UserB=\n fr.aneo.armonik.api.grpc.v1.auth\xaa\x02\x18\x41rmoniK.Api.gRPC.V1.Authb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'auth_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n fr.aneo.armonik.api.grpc.v1.auth\252\002\030ArmoniK.Api.gRPC.V1.Auth'
  _globals['_USER']._serialized_start=47
  _globals['_USER']._serialized_end=107
  _globals['_GETCURRENTUSERREQUEST']._serialized_start=109
  _globals['_GETCURRENTUSERREQUEST']._serialized_end=132
  _globals['_GETCURRENTUSERRESPONSE']._serialized_start=134
  _globals['_GETCURRENTUSERRESPONSE']._serialized_end=204
# @@protoc_insertion_point(module_scope)
