# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: applications_fields.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x61pplications_fields.proto\x12 armonik.api.grpc.v1.applications\"_\n\x13\x41pplicationRawField\x12H\n\x05\x66ield\x18\x01 \x01(\x0e\x32\x39.armonik.api.grpc.v1.applications.ApplicationRawEnumField\"o\n\x10\x41pplicationField\x12R\n\x11\x61pplication_field\x18\x01 \x01(\x0b\x32\x35.armonik.api.grpc.v1.applications.ApplicationRawFieldH\x00\x42\x07\n\x05\x66ield*\xe4\x01\n\x17\x41pplicationRawEnumField\x12*\n&APPLICATION_RAW_ENUM_FIELD_UNSPECIFIED\x10\x00\x12#\n\x1f\x41PPLICATION_RAW_ENUM_FIELD_NAME\x10\x01\x12&\n\"APPLICATION_RAW_ENUM_FIELD_VERSION\x10\x02\x12(\n$APPLICATION_RAW_ENUM_FIELD_NAMESPACE\x10\x03\x12&\n\"APPLICATION_RAW_ENUM_FIELD_SERVICE\x10\x04\x42M\n(fr.aneo.armonik.api.grpc.v1.applications\xaa\x02 ArmoniK.Api.gRPC.V1.Applicationsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'applications_fields_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n(fr.aneo.armonik.api.grpc.v1.applications\252\002 ArmoniK.Api.gRPC.V1.Applications'
  _globals['_APPLICATIONRAWENUMFIELD']._serialized_start=274
  _globals['_APPLICATIONRAWENUMFIELD']._serialized_end=502
  _globals['_APPLICATIONRAWFIELD']._serialized_start=63
  _globals['_APPLICATIONRAWFIELD']._serialized_end=158
  _globals['_APPLICATIONFIELD']._serialized_start=160
  _globals['_APPLICATIONFIELD']._serialized_end=271
# @@protoc_insertion_point(module_scope)
