# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tasks_service.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import tasks_common_pb2 as tasks__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13tasks_service.proto\x12\x19\x61rmonik.api.grpc.v1.tasks\x1a\x12tasks_common.proto2\xa6\x06\n\x05Tasks\x12h\n\tListTasks\x12+.armonik.api.grpc.v1.tasks.ListTasksRequest\x1a,.armonik.api.grpc.v1.tasks.ListTasksResponse\"\x00\x12x\n\x11ListTasksDetailed\x12+.armonik.api.grpc.v1.tasks.ListTasksRequest\x1a\x34.armonik.api.grpc.v1.tasks.ListTasksDetailedResponse\"\x00\x12\x62\n\x07GetTask\x12).armonik.api.grpc.v1.tasks.GetTaskRequest\x1a*.armonik.api.grpc.v1.tasks.GetTaskResponse\"\x00\x12n\n\x0b\x43\x61ncelTasks\x12-.armonik.api.grpc.v1.tasks.CancelTasksRequest\x1a..armonik.api.grpc.v1.tasks.CancelTasksResponse\"\x00\x12o\n\x0cGetResultIds\x12..armonik.api.grpc.v1.tasks.GetResultIdsRequest\x1a/.armonik.api.grpc.v1.tasks.GetResultIdsResponse\x12\x83\x01\n\x12\x43ountTasksByStatus\x12\x34.armonik.api.grpc.v1.tasks.CountTasksByStatusRequest\x1a\x35.armonik.api.grpc.v1.tasks.CountTasksByStatusResponse\"\x00\x12n\n\x0bSubmitTasks\x12-.armonik.api.grpc.v1.tasks.SubmitTasksRequest\x1a..armonik.api.grpc.v1.tasks.SubmitTasksResponse\"\x00\x42?\n!fr.aneo.armonik.api.grpc.v1.tasks\xaa\x02\x19\x41rmoniK.Api.gRPC.V1.Tasksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tasks_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!fr.aneo.armonik.api.grpc.v1.tasks\252\002\031ArmoniK.Api.gRPC.V1.Tasks'
  _globals['_TASKS']._serialized_start=71
  _globals['_TASKS']._serialized_end=877
# @@protoc_insertion_point(module_scope)
