# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: results_service.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import objects_pb2 as objects__pb2
from ..common import results_common_pb2 as results__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15results_service.proto\x12\x1b\x61rmonik.api.grpc.v1.results\x1a\robjects.proto\x1a\x14results_common.proto2\x83\x0b\n\x07Results\x12r\n\x0bListResults\x12/.armonik.api.grpc.v1.results.ListResultsRequest\x1a\x30.armonik.api.grpc.v1.results.ListResultsResponse\"\x00\x12l\n\tGetResult\x12-.armonik.api.grpc.v1.results.GetResultRequest\x1a..armonik.api.grpc.v1.results.GetResultResponse\"\x00\x12y\n\x0eGetOwnerTaskId\x12\x32.armonik.api.grpc.v1.results.GetOwnerTaskIdRequest\x1a\x33.armonik.api.grpc.v1.results.GetOwnerTaskIdResponse\x12\x90\x01\n\x15\x43reateResultsMetaData\x12\x39.armonik.api.grpc.v1.results.CreateResultsMetaDataRequest\x1a:.armonik.api.grpc.v1.results.CreateResultsMetaDataResponse\"\x00\x12x\n\rCreateResults\x12\x31.armonik.api.grpc.v1.results.CreateResultsRequest\x1a\x32.armonik.api.grpc.v1.results.CreateResultsResponse\"\x00\x12\x83\x01\n\x10UploadResultData\x12\x34.armonik.api.grpc.v1.results.UploadResultDataRequest\x1a\x35.armonik.api.grpc.v1.results.UploadResultDataResponse\"\x00(\x01\x12\x89\x01\n\x12\x44ownloadResultData\x12\x36.armonik.api.grpc.v1.results.DownloadResultDataRequest\x1a\x37.armonik.api.grpc.v1.results.DownloadResultDataResponse\"\x00\x30\x01\x12\x84\x01\n\x11\x44\x65leteResultsData\x12\x35.armonik.api.grpc.v1.results.DeleteResultsDataRequest\x1a\x36.armonik.api.grpc.v1.results.DeleteResultsDataResponse\"\x00\x12\x84\x01\n\x11ImportResultsData\x12\x35.armonik.api.grpc.v1.results.ImportResultsDataRequest\x1a\x36.armonik.api.grpc.v1.results.ImportResultsDataResponse\"\x00\x12w\n\x17GetServiceConfiguration\x12\x1a.armonik.api.grpc.v1.Empty\x1a@.armonik.api.grpc.v1.results.ResultsServiceConfigurationResponse\x12u\n\x0cWatchResults\x12/.armonik.api.grpc.v1.results.WatchResultRequest\x1a\x30.armonik.api.grpc.v1.results.WatchResultResponse(\x01\x30\x01\x42\x43\n#fr.aneo.armonik.api.grpc.v1.results\xaa\x02\x1b\x41rmoniK.Api.gRPC.V1.Resultsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'results_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#fr.aneo.armonik.api.grpc.v1.results\252\002\033ArmoniK.Api.gRPC.V1.Results'
  _globals['_RESULTS']._serialized_start=92
  _globals['_RESULTS']._serialized_end=1503
# @@protoc_insertion_point(module_scope)
