# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: partitions_filters.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import filters_common_pb2 as filters__common__pb2
from . import partitions_fields_pb2 as partitions__fields__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18partitions_filters.proto\x12\x1e\x61rmonik.api.grpc.v1.partitions\x1a\x14\x66ilters_common.proto\x1a\x17partitions_fields.proto\"\xcf\x02\n\x0b\x46ilterField\x12=\n\x05\x66ield\x18\x01 \x01(\x0b\x32..armonik.api.grpc.v1.partitions.PartitionField\x12:\n\rfilter_string\x18\x02 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterStringH\x00\x12:\n\rfilter_number\x18\x03 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterNumberH\x00\x12<\n\x0e\x66ilter_boolean\x18\x04 \x01(\x0b\x32\".armonik.api.grpc.v1.FilterBooleanH\x00\x12\x38\n\x0c\x66ilter_array\x18\x05 \x01(\x0b\x32 .armonik.api.grpc.v1.FilterArrayH\x00\x42\x11\n\x0fvalue_condition\"F\n\nFiltersAnd\x12\x38\n\x03\x61nd\x18\x01 \x03(\x0b\x32+.armonik.api.grpc.v1.partitions.FilterField\"A\n\x07\x46ilters\x12\x36\n\x02or\x18\x01 \x03(\x0b\x32*.armonik.api.grpc.v1.partitions.FiltersAndBI\n&fr.aneo.armonik.api.grpc.v1.partitions\xaa\x02\x1e\x41rmoniK.Api.gRPC.V1.Partitionsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'partitions_filters_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n&fr.aneo.armonik.api.grpc.v1.partitions\252\002\036ArmoniK.Api.gRPC.V1.Partitions'
  _globals['_FILTERFIELD']._serialized_start=108
  _globals['_FILTERFIELD']._serialized_end=443
  _globals['_FILTERSAND']._serialized_start=445
  _globals['_FILTERSAND']._serialized_end=515
  _globals['_FILTERS']._serialized_start=517
  _globals['_FILTERS']._serialized_end=582
# @@protoc_insertion_point(module_scope)
