# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: worker_common.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import objects_pb2 as objects__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13worker_common.proto\x12\x1a\x61rmonik.api.grpc.v1.worker\x1a\robjects.proto\"\xa7\x02\n\x0eProcessRequest\x12\x1b\n\x13\x63ommunication_token\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x0f\n\x07task_id\x18\x03 \x01(\t\x12\x36\n\x0ctask_options\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12\x1c\n\x14\x65xpected_output_keys\x18\x05 \x03(\t\x12\x12\n\npayload_id\x18\x06 \x01(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x07 \x03(\t\x12\x13\n\x0b\x64\x61ta_folder\x18\x08 \x01(\t\x12\x39\n\rconfiguration\x18\t \x01(\x0b\x32\".armonik.api.grpc.v1.Configuration\";\n\x0cProcessReply\x12+\n\x06output\x18\x01 \x01(\x0b\x32\x1b.armonik.api.grpc.v1.Output\"\x9a\x01\n\x10HealthCheckReply\x12J\n\x06status\x18\x01 \x01(\x0e\x32:.armonik.api.grpc.v1.worker.HealthCheckReply.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x42\x41\n\"fr.aneo.armonik.api.grpc.v1.worker\xaa\x02\x1a\x41rmoniK.Api.gRPC.V1.Workerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'worker_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"fr.aneo.armonik.api.grpc.v1.worker\252\002\032ArmoniK.Api.gRPC.V1.Worker'
  _globals['_PROCESSREQUEST']._serialized_start=67
  _globals['_PROCESSREQUEST']._serialized_end=362
  _globals['_PROCESSREPLY']._serialized_start=364
  _globals['_PROCESSREPLY']._serialized_end=423
  _globals['_HEALTHCHECKREPLY']._serialized_start=426
  _globals['_HEALTHCHECKREPLY']._serialized_end=580
  _globals['_HEALTHCHECKREPLY_SERVINGSTATUS']._serialized_start=522
  _globals['_HEALTHCHECKREPLY_SERVINGSTATUS']._serialized_end=580
# @@protoc_insertion_point(module_scope)
